/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.jira.test;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Hashtable;

import org.apache.xmlrpc.XmlRpcException;
import org.apache.xmlrpc.XmlRpcRequest;
import org.apache.xmlrpc.XmlRpcRequestConfig;
import org.apache.xmlrpc.client.XmlRpcAhcTransportFactory;
import org.apache.xmlrpc.client.XmlRpcClient;
import org.apache.xmlrpc.client.XmlRpcClientConfigImpl;
import org.apache.xmlrpc.client.XmlRpcClientRequestImpl;
import org.codehaus.swizzle.jira.MapObject;
import org.junit.After;
import org.junit.Before;
import org.sonatype.guice.bean.containers.InjectedTestCase;
import org.sonatype.jira.JiraXmlRpcProxy;
import org.sonatype.tests.http.runner.api.SuiteConfiguration;
import org.sonatype.tests.http.runner.api.SuiteConfigurator;
import org.sonatype.tests.http.runner.junit.DefaultSuiteConfiguration;
import org.sonatype.tests.http.server.api.ServerProvider;

public abstract class JiraProxyTestCase
    extends InjectedTestCase
    implements SuiteConfiguration
{

    private JiraXmlRpcProxy handler;
    private final DefaultSuiteConfiguration delegate = new DefaultSuiteConfiguration()
    {
    };

    @Override
    public void setConfigurator( final SuiteConfigurator configurator )
    {
        delegate.setConfigurator( configurator );
    }

    public void configureProvider( final ServerProvider provider )
    {
        delegate.configureProvider( provider );
    }

    public String url()
    {
        return delegate.url();
    }

    public ServerProvider provider()
    {
        return delegate.provider();
    }

    public String url( final String path, final String... parts )
    {
        return delegate.url( path, parts );
    }

    public SuiteConfigurator configurator()
    {
        return delegate.configurator();
    }

    protected XmlRpcRequest request( final String method, final String... parameters )
        throws Exception
    {
        return new XmlRpcClientRequestImpl( rpcConfig(), "jira1." + method, parameters );
    }

    private XmlRpcRequestConfig rpcConfig()
        throws Exception
    {
        XmlRpcClientConfigImpl cfg = new XmlRpcClientConfigImpl();
        cfg.setServerURL( rcpUrl() );
        return cfg;
    }

    private URL rcpUrl()
        throws Exception
    {
        return new URL( provider().getUrl().toString() + "/rpc/xmlrpc" );
    }

    protected Object execute( final String method, final String... parameters )
        throws XmlRpcException, MalformedURLException, Exception
    {
        Object response = client().execute( request( method, parameters ) );
        return response;
    }

    protected XmlRpcClient client()
        throws MalformedURLException
    {
        XmlRpcClientConfigImpl clientConfig = new XmlRpcClientConfigImpl();
        clientConfig.setServerURL( provider().getUrl() );
        XmlRpcClient client = new XmlRpcClient();
        client.setTransportFactory( new XmlRpcAhcTransportFactory( client ) );
        client.setConfig( clientConfig );
        return client;
    }

    void handler( final JiraXmlRpcProxy handler )
    {
        this.handler = handler;
    }

    @Before
    @Override
    public void setUp()
        throws Exception
    {
        super.setUp();
        delegate.before();
    }

    @After
    @Override
    public void tearDown()
        throws Exception
    {
        super.tearDown();
        delegate.after();
    }

    public JiraXmlRpcProxy handler()
    {
        return this.handler;
    }

    @SuppressWarnings( { "unchecked", "rawtypes" } )
    protected Hashtable<?, ?> convert( final MapObject obj )
    {
        return new Hashtable( obj.toMap() );
    }
}
