/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.jira.test;

import java.io.File;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

import javax.inject.Inject;

import org.sonatype.inject.Sisu;
import org.sonatype.jira.AttachmentHandler;
import org.sonatype.jira.JiraXmlRpcProxy;
import org.sonatype.jira.recorder.RecordingProxy;
import org.sonatype.tests.http.server.api.ServerProvider;
import org.sonatype.tests.http.server.jetty.impl.JettyServerProvider;

public class ProxyConfigurator
implements JiraProxySuiteConfigurator
{

    private URL url;

    private final JiraXmlRpcProxy handler;

    private final Method method;

    @Inject
    private List<AttachmentHandler> handlers;

    public ProxyConfigurator( final Method method, final JiraXmlRpcProxy proxy, final String url )
    {
        Sisu.inject( this );
        this.method = method;
        this.handler = proxy;
        try
        {
            this.url = new URL( url );
        }
        catch ( MalformedURLException e )
        {
            throw new RuntimeException( e );
        }
    }

    @Override
    public ServerProvider provider()
    {
        JettyServerProvider provider;
        try
        {
            provider = new JettyServerProvider()
            {

                @Override
                public void stop()
                throws Exception
                {
                    if ( handler instanceof RecordingProxy && ( (RecordingProxy) handler ).isRecording() )
                    {
                        String logPath =
                            String.format( "target/%s#%s.xml", method.getDeclaringClass().getName(), method.getName() );
                        ( (RecordingProxy) handler ).write( new File( logPath ) );
                    }
                    super.stop();
                }

            };
        }
        catch ( Exception e )
        {
            throw new RuntimeException( e );
        }
        provider.addServlet( new JiraXmlRpcTestServlet( handler, url, handlers ) );
        return provider;
    }

    @Override
    public String getName()
    {
        return "PROXY::" + url.toString();
    }

    @Override
    public JiraXmlRpcProxy getHandler()
    {
        return handler;
    }

}
